"""
Validate that the annotations match the corresponding docs text field.
"""

import argparse
from typing import cast

from pyspark.sql import SparkSession
from pyspark.sql.functions import col

from trend_etl.functions.index import full_table_name, annotation_matches_text
import trend_etl.schema.args as arguments

PROGRAM = "validate_nlp_annotations"
OUTPUT_PATH = "dbfs:/FileStore/charlie/failed_annotation_matches"


def main() -> None:
    """
    Entry point for task
    """
    parser = argparse.ArgumentParser(description=f"Arguments for Trend ETL/{PROGRAM}.")

    parser.add_argument(
        "--target-catalog",
        dest="target_catalog",
        required=True,
        choices=arguments.catalogs,
        help="The unit catalog to use",
    )

    args = parser.parse_args()

    spark = cast(SparkSession, SparkSession.builder.appName(PROGRAM).getOrCreate())

    docs_table = full_table_name(
        args.target_catalog,
        arguments.schemas[arguments.schemas_silver],
        arguments.tables[arguments.tables_docs],
    )
    docs = spark.read.format("delta").table(docs_table)

    nlp_results_table = full_table_name(
        args.target_catalog,
        arguments.schemas[arguments.schemas_silver],
        arguments.tables[arguments.tables_historical_prod_nlp_results],
    )
    nlp_results = (
        spark.read.format("delta")
        .table(nlp_results_table)
        .where(col("model_name") == "csrl")
        .where(col("model_version") == 1)
    )

    joined = nlp_results.join(docs.selectExpr("doc_uuid", "texts"), "doc_uuid")

    print(f"nlp results count is {nlp_results.count()}")
    print(f"joined count is {joined.count()}")

    failed_matches = joined.where(
        ~annotation_matches_text(col("texts.text"), col("annotation"))
    )

    (
        failed_matches.repartition(1)
        .write.mode("overwrite")
        .format("delta")
        .save(OUTPUT_PATH)
    )
